/*
 * UART.c
 * For sending and receiving data over UART
 * interface with RS-232
 */
#include "msp.h"
#include "UART.h"
#include "delay.h"
#include "temp.h"
#include <math.h>

#define UCBRS   0//(0x20)  // 0x00
#define UCBRF   10//(8)     // 10
#define UCBRX   1//(6)     // 1

int buf[3];

uint16_t threshold;

void UART_init()
{
    // Configure UART pins
    P1->SEL0 |= BIT2 | BIT3;                // set 2-UART pin as secondary function

    // Configure UART
    EUSCI_A0->CTLW0 |= EUSCI_A_CTLW0_SWRST; // Put eUSCI in reset
    EUSCI_A0->CTLW0 = EUSCI_A_CTLW0_SWRST | // Remain eUSCI in reset
            EUSCI_B_CTLW0_SSEL__SMCLK;      // Configure eUSCI clock source for SMCLK

    // Baud Rate calculation for 3MHz
    // 3000000/(115200) = 26.041667
    // Fractional portion = 0.041667
    // User's Guide Table 21-4: UCBRSx = 0x00
    // UCBRx = int (26.041667 / 16) = 1  ? 10
    // UCBRFx = int (((26.041667/16)-1)*16) = 10  ? 13

    //BRW = UCBRx
    EUSCI_A0->BRW = UCBRX;                      // Using baud rate calculator
    //MCTLW = UCBRSx[15:8] UCBRFx[7:4] -[3:1] OS[0]
    EUSCI_A0->MCTLW = (UCBRF << EUSCI_A_MCTLW_BRF_OFS) |
                      (UCBRS << EUSCI_A_MCTLW_BRS_OFS) |
                      EUSCI_A_MCTLW_OS16;
    EUSCI_A0->CTLW0 &= ~EUSCI_A_CTLW0_SWRST; // Initialize eUSCI
    EUSCI_A0->IFG &= ~EUSCI_A_IFG_RXIFG;    // Clear eUSCI RX interrupt flag
    EUSCI_A0->IE |= EUSCI_A_IE_RXIE;        // Enable USCI_A0 RX interrupt

    // Enable global interrupt
    __enable_irq();

    // Enable eUSCIA0 interrupt in NVIC module
    NVIC->ISER[0] |= 1 << ((EUSCIA0_IRQn) & 31);

    threshold = DEFAULT_THRESHOLD;

}

void sendUARTint(int num)
{
    signed char toSend[10], i;
    int remainder = num;
    for(i = 0; remainder; i++)
    {
        toSend[i] = remainder % 10  + ZERO_CHR;
        remainder /= 10;
    }
    for(i--; i >= 0; i--)
    {
        sendUARTchar(toSend[i]);
    }
}

void sendUARTstr(char str[])
{
    for(; *str; str++)
        sendUARTchar(*str);
}

void sendUARTchar(char data)
{
    EUSCI_A0->TXBUF = data;
    delay_us(106);
}

uint16_t getThreshold()
{
    return threshold;
}

// UART interrupt service routine
void EUSCIA0_IRQHandler(void)
{
    if (EUSCI_A0->IFG & EUSCI_A_IFG_RXIFG)
    {
        // Check if the TX buffer is empty first
        while(!(EUSCI_A0->IFG & EUSCI_A_IFG_TXIFG));

        if(EUSCI_A0->RXBUF >= ZERO_CHR && EUSCI_A0->RXBUF <= NINE_CHR)
        {
            // Echo the received character back
            EUSCI_A0->TXBUF = EUSCI_A0->RXBUF;
            buf[0] = buf[1];
            buf[1] = buf[2];
            buf[2] = EUSCI_A0->RXBUF - ZERO_CHR;
        }
        else if(EUSCI_A0->RXBUF == RETURN_CHR)
        {
            threshold = (100 * buf[0] + 10 * buf[1] + buf[2]);

            if(threshold > 120)
                threshold = 120;

            buf[0] = 0;
            buf[1] = 0;
            buf[2] = 0;
//            EUSCI_A0->TXBUF = NEW_LINE_CHR;
//            for(i = 0; i < 1000; ++i); // delay
//            EUSCI_A0->TXBUF = RETURN_CHR;
        }
    }
}

